﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Xml;
using Microsoft.Web.CommandUI;

namespace ContextualTabWebPart.ContextualTabWebPart
{
    /// <summary>
    /// This is a sample web part which opens a contextual tab on the ribbon in Microsoft SharePoint Foundation 2010.
    /// http://blogs.msdn.com/b/sharepointdeveloperdocs/archive/2010/01/28/how-to-create-a-web-part-with-a-contextual-tab.aspx
    /// </summary>
    [ToolboxItemAttribute(false)]
    public class ContextualTabWebPart : WebPart, IWebPartPageComponentProvider
    {
        static bool _added = false;
        
        #region private strings

        /*
         * These two variables define the contextual tab and the group template.
         * 
         * For a more in-depth discussion on defining a tab and group template,
         * see How to Add a Tab to the Server Ribbon in SharePoint Foundation:
         * http://blogs.msdn.com/sharepointdeveloperdocs/archive/2009/12/07/SharePointFoundationHowToAddTabToRibbon.aspx
         * 
         * In the contextualTab string, you will notice a ContextualGroup element.
         * This element defines the following Tab element as a contextual tab.
         * The Color attribute defines the color of the tab when it renders.
         * The Id and ContextualGroupId are simply unique identifiers for the group.
         * The Sequence attribute defines where the contextual tab will render.
         */

        private string contextualTab = @"
   <ContextualGroup Color=""Magenta"" 
     Command=""CustomContextualTab.EnableContextualGroup"" 
     Id=""Ribbon.CustomContextualTabGroup"" 
     Title=""Custom Contextual Tab Group"" 
     Sequence=""502"" 
     ContextualGroupId=""CustomContextualTabGroup"">
          <Tab 
              Id=""Ribbon.CustomTabExample"" 
              Title=""My Custom Tab"" 
              Description=""This holds my custom commands!"" 
              Command=""CustomContextualTab.EnableCustomTab"" 
              Sequence=""501"">
            <Scaling
              Id=""Ribbon.CustomTabExample.Scaling"">
              <MaxSize
                Id=""Ribbon.CustomTabExample.MaxSize"" 
                GroupId=""Ribbon.CustomTabExample.CustomGroupExample"" 
                Size=""OneLargeTwoMedium""/>
              <Scale 
                Id=""Ribbon.CustomTabExample.Scaling.CustomTabScaling""
                GroupId=""Ribbon.CustomTabExample.CustomGroupExample"" 
                Size=""OneLargeTwoMedium"" />
            </Scaling>
            <Groups Id=""Ribbon.CustomTabExample.Groups"">
              <Group 
                Id=""Ribbon.CustomTabExample.CustomGroupExample"" 
                Description=""This is a custom group!"" 
                Title=""Custom Group"" 
                Command=""CustomContextualTab.EnableCustomGroup"" 
                Sequence=""52"" 
                Template=""Ribbon.Templates.CustomTemplateExample"">
                <Controls 
                  Id=""Ribbon.CustomTabExample.CustomGroupExample.Controls"">
                  <Button 
                    Id=""Ribbon.CustomTabExample.CustomGroupExample.HelloWorld"" 
                    Command=""CustomContextualTab.HelloWorldCommand"" 
                    Sequence=""15"" 
                    Description=""Says hello to the World!"" 
                    LabelText=""Hello, World!"" 
                    TemplateAlias=""cust1""/>
                  <Button 
                    Id=""Ribbon.CustomTabExample.CustomGroupExample.GoodbyeWorld"" 
                    Command=""CustomContextualTab.GoodbyeWorldCommand"" 
                    Sequence=""17"" 
                    Description=""Says good-bye to the World!"" 
                    LabelText=""Good-bye, World!"" 
                    TemplateAlias=""cust2""/>
                </Controls>
              </Group>
            </Groups>
          </Tab>
   </ContextualGroup>";

        private string contextualTabTemplate = @"
          <GroupTemplate Id=""Ribbon.Templates.CustomTemplateExample"">
            <Layout 
              Title=""OneLargeTwoMedium"" LayoutTitle=""OneLargeTwoMedium"">
              <Section Alignment=""Top"" Type=""OneRow"">
                <Row>
                  <ControlRef DisplayMode=""Large"" TemplateAlias=""cust1"" />
                </Row>
              </Section>
              <Section Alignment=""Top"" Type=""TwoRow"">
                <Row>
                  <ControlRef DisplayMode=""Medium"" TemplateAlias=""cust2"" />
                </Row>
                <Row>
                  <ControlRef DisplayMode=""Medium"" TemplateAlias=""cust3"" />
                </Row>
              </Section>
            </Layout>
          </GroupTemplate>";

        /// <summary>
        /// DelayScript contains EcmaScript (JavaScript, JScript) that adds and registers your custom page component.

        /// The _addCustomPageComponent method loops through the global custom web part array (g_customWebPartIds) and adds each custom page component to the page manager.
        /// Every Web Part has a page component, and you will use the Web Part's page component id when you create a custom page component object.
        /// This is necessary to associate the Web Part with the page component to enable contextually switching to your custom ribbon tab.

        /// The _registerCustomPageComponent method registers your EcmaScript file when the page loads.
        /// </summary>
        private string delayScript = @"
            function _addCustomPageComponent()
            {
                for (var i = 0; i < g_customWebPartIds.length; i++)
                {
                        SP.Ribbon.PageManager.get_instance().addPageComponent(new ContextualTabWebPart.CustomPageComponent(g_customWebPartIds[i]));
                }
            }

            function _registerCustomPageComponent()
            {
                SP.SOD.registerSod(""CustomContextualTabPageComponent.js"", ""/_layouts/CustomContextualTabPageComponent.js"");
                var isDefined = ""undefined"";
                try
                {
                    isDefined = typeof(ContextualTabWebPart.CustomPageComponent);
                }
                catch(e)
                {
                }
                EnsureScript(""CustomContextualTabPageComponent.js"",isDefined, _addCustomPageComponent);
            }
            SP.SOD.executeOrDelayUntilScriptLoaded(_registerCustomPageComponent, ""sp.ribbon.js"");";

        #endregion

        #region methods
        /// <summary>
        /// Registers the contextual tab and custom templates with the ribbon.
        /// The Microsoft.Web.CommandUI.Ribbon.RegisterDataExtension method will be used to register the ribbon extensions.
        /// RegisterDataExtension tells the ribbon where to load the XML passed into it.
        /// We will use the ids of the ContextualTabs and Templates elements in CMDUI.xml
        /// </summary>
        private void AddContextualTab()
        {
            //Gets the current instance of the ribbon on the page.
            Microsoft.Web.CommandUI.Ribbon ribbon = SPRibbon.GetCurrent(this.Page);
            ribbon.MakeTabAvailable("Ribbon.CustomTabExample");

            //Prepares an XmlDocument object used to load the ribbon extensions.
            XmlDocument ribbonExtensions = new XmlDocument();

            //Load the contextual tab XML and register the ribbon extension.
            ribbonExtensions.LoadXml(this.contextualTab);
            ribbon.RegisterDataExtension(ribbonExtensions.FirstChild, "Ribbon.ContextualTabs._children");

            //Load the custom templates and register the ribbon extension.
            ribbonExtensions.LoadXml(this.contextualTabTemplate);
            ribbon.RegisterDataExtension(ribbonExtensions.FirstChild, "Ribbon.Templates._children");
        }
        #endregion

        #region events

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            _added = false;
        }
        
        /// <summary>
        /// Add elements to the ribbon before it is rendered on the page.
        /// Call the AddContextualTab method and register the DelayScript with the ClientScriptManager.
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            ClientScriptManager csm = this.Page.ClientScript;
            string componentId = SPRibbon.GetWebPartPageComponentId(this);

            // if this is the first instance of the custom web part,
            //we need to add the contextual tab to the ribbon
            if (!_added)
            {
                this.AddContextualTab();
                _added = true;
            }

            // we need to create an array which will store the IDs of all instances of our custom web part
            csm.RegisterClientScriptBlock(
                this.GetType(), "DeclareCustomWebPartArray", "var g_customWebPartIds = new Array();", true);

            // add this webpart's ID to our array
            csm.RegisterClientScriptBlock(
                this.GetType(), "AddCustomWebPartId" + componentId, "g_customWebPartIds.push('" + componentId + "');", true);
            
            csm.RegisterClientScriptBlock(this.GetType(), "ContextualTabWebPart", this.delayScript, true);
        }
        #endregion

        #region interface
        /// <summary>
        /// This interface lets the ribbon know when a Web Part has been selected and which contextual group and tab to show.

        /// When you add the contextual group, you send it the id of the contextual group, the VisibilityContext, and the Command.
        /// The id maps to the Id property of the ContextualGroup element in the ribbon XML.
        /// The VisibilityContext is used to group controls in order to show or hide them.
        /// The Command parameter maps to the Command of the ContextualGroup element in the ribbon XML.

        /// You will also add the tab that was defined in the ribbon XML.
        /// You only need to pass in the Id and the VisibilityContext parameters for the tab. 
        /// </summary>
        WebPartContextualInfo IWebPartPageComponentProvider.WebPartContextualInfo
        {
            get
            {
                WebPartContextualInfo info = new WebPartContextualInfo();
                WebPartRibbonContextualGroup contextualGroup = new WebPartRibbonContextualGroup();
                WebPartRibbonTab ribbonTab = new WebPartRibbonTab();

                //Create the contextual group object and initialize its values.
                contextualGroup.Id = "Ribbon.CustomContextualTabGroup";
                contextualGroup.Command = "CustomContextualTab.EnableContextualGroup";
                contextualGroup.VisibilityContext = "CustomContextualTab.CustomVisibilityContext";

                //Create the tab object and initialize its values.
                ribbonTab.Id = "Ribbon.CustomTabExample";
                ribbonTab.VisibilityContext = "CustomContextualTab.CustomVisibilityContext";

                //Add the contextual group and tab to the WebPartContextualInfo.
                info.ContextualGroups.Add(contextualGroup);
                info.Tabs.Add(ribbonTab);
                info.PageComponentId = SPRibbon.GetWebPartPageComponentId(this);

                return info;
            }
        }
        #endregion
    }
}
